package com.zanxiang.game.back.serve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.back.serve.dao.mapper.TempCallbackMapper;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentMiniGameUser;
import com.zanxiang.game.back.serve.pojo.entity.TempCallback;
import com.zanxiang.game.back.serve.service.IGameTencentMiniGameUserService;
import com.zanxiang.game.back.serve.service.ITempCallbackService;
import com.zanxiang.module.util.DateUtil;
import com.zanxiang.module.util.URIUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

/**
 * @author : lingfeng
 * @time : 2025-03-06
 * @description : 临时表
 */
@Slf4j
@Service
public class TempCallbackServiceImpl extends ServiceImpl<TempCallbackMapper, TempCallback> implements ITempCallbackService {

    @Autowired
    private IGameTencentMiniGameUserService gameTencentMiniGameUserService;

    @Override
    public void checkCallBackUrl() {
        long current = 1;
        long size = 5000;
        long pages;
        do {
            Page<GameTencentMiniGameUser> miniGameUserPage = gameTencentMiniGameUserService.page(Page.of(current, size),
                    new LambdaQueryWrapper<GameTencentMiniGameUser>()
                            .select(GameTencentMiniGameUser::getId, GameTencentMiniGameUser::getWechatOpenid,
                                    GameTencentMiniGameUser::getWechatOpenid)
                            .eq(GameTencentMiniGameUser::getGameId, 35L)
                            .eq(GameTencentMiniGameUser::getBackType, "BACK_API")
                            .isNull(GameTencentMiniGameUser::getCallBackUrl)
                            .orderByDesc(GameTencentMiniGameUser::getCreateTime));
            pages = miniGameUserPage.getPages();
            log.error("第 {} 页正在处理-------> , 共 {} 页", current, pages);
            this.updateCallBackUrlBatch(miniGameUserPage.getRecords());
            log.error("第 {} 处理完成 =========> , 共 {} 页", current, pages);
        } while (++current <= pages);
    }

    private void updateCallBackUrlBatch(List<GameTencentMiniGameUser> records) {
        if (CollectionUtils.isEmpty(records)) {
            return;
        }
        records.forEach(r -> {
            String callBackUrl = this.getCallBackUrl(r.getGameId(), r.getWechatOpenid(), r.getClickId(), r.getRegisterTime());
            if (Strings.isBlank(callBackUrl)) {
                return;
            }
            gameTencentMiniGameUserService.update(new LambdaUpdateWrapper<GameTencentMiniGameUser>()
                    .set(GameTencentMiniGameUser::getCallBackUrl, callBackUrl)
                    .eq(GameTencentMiniGameUser::getId, r.getId())
            );
        });
    }

    private String getCallBackUrl(Long gameId, String wechatOpenid, String clickId, LocalDateTime registerTime) {
        try {
            //1. 精准匹配, openId 必须匹配, requestId, clickId, impressionId, 有一个匹配上即可
            TempCallback tempCallback = super.getOne(new LambdaQueryWrapper<TempCallback>()
                    .eq(TempCallback::getGameId, gameId)
                    .eq(TempCallback::getWechatOpenid, wechatOpenid)
                    .and(qw -> qw.eq(TempCallback::getImpressionId, clickId)
                            .or().eq(TempCallback::getRequestId, clickId)
                            .or().eq(TempCallback::getClickId, clickId))
                    .orderByDesc(TempCallback::getClickTime)
                    .last("limit 1"));
            //2. 模糊匹配, 分别使用 requestId, clickId, impressionId 匹配, 有一个匹配上即可, 加上注册时间距离最近条件
            if (tempCallback == null) {
                tempCallback = super.getOne(new LambdaQueryWrapper<TempCallback>()
                        .eq(TempCallback::getGameId, gameId)
                        .and(qw -> qw.eq(TempCallback::getImpressionId, clickId)
                                .or().eq(TempCallback::getRequestId, clickId)
                                .or().eq(TempCallback::getClickId, clickId))
                        .le(TempCallback::getClickTime, DateUtil.localDateTimeToSecond(registerTime))
                        .orderByDesc(TempCallback::getClickTime)
                        .last("limit 1"));
            }
            //3. 兜底逻辑, 只用openId匹配
            if (tempCallback == null) {
                tempCallback = super.getOne(new LambdaQueryWrapper<TempCallback>()
                        .eq(TempCallback::getGameId, gameId)
                        .eq(TempCallback::getWechatOpenid, wechatOpenid)
                        .orderByDesc(TempCallback::getClickTime)
                        .last("limit 1"));
            }
            String callBack = Optional.ofNullable(tempCallback)
                    .map(TempCallback::getCallback)
                    .orElse(null);
            if (Strings.isBlank(callBack)) {
                log.error("匹配监测链接为空, wechatOpenid : {}, clickId : {}", wechatOpenid, clickId);
                return null;
            }
            //返回匹配回传地址
            return URIUtil.decodeURIComponent(callBack);
        } catch (Exception e) {
            log.error("匹配监测链接异常, wechatOpenid : {}, clickId : {}", wechatOpenid, clickId);
        }
        return null;
    }
}
